#include "hal_data.h"
#include "da16200_AT.h"
#include "common_utils.h"
#include <stdio.h>

/* TODO: Enable if you want to open ZMOD4XXX */
#define G_ZMOD4XXX_SENSOR0_IRQ_ENABLE   (1)

typedef enum e_demo_sequence
{
    DEMO_SEQUENCE_1 = (1),
    DEMO_SEQUENCE_2,
    DEMO_SEQUENCE_3,
    DEMO_SEQUENCE_4,
    DEMO_SEQUENCE_5,
    DEMO_SEQUENCE_6,
    DEMO_SEQUENCE_7,
    DEMO_SEQUENCE_8,
    DEMO_SEQUENCE_9,
} demo_sequence_t;

typedef enum e_demo_callback_status
{
    DEMO_CALLBACK_STATUS_WAIT = (0),
    DEMO_CALLBACK_STATUS_SUCCESS,
    DEMO_CALLBACK_STATUS_REPEAT,
} demo_callback_status_t;

uint8_t update_flag = 0;
char tvoc_title[30] = "TVOC:";
char tvoc_value_str[10];
char eco2_title[30] = "ECO2:";
char eco2_value_str[10];
char iaq_title[30] = "IAQ:";
char iaq_value_str[10];
char air_str[100];
char mqtt_head[] = "AT+NWMQMSG=";

void g_comms_i2c_bus0_quick_setup(void);
void g_zmod4xxx_sensor0_quick_setup(void);
void mcu_hw_init(void);
void start_demo(void);
void demo_err(void);
void mqtt_msg_send(void);

static volatile demo_callback_status_t  gs_i2c_callback_status = DEMO_CALLBACK_STATUS_WAIT;
#if G_ZMOD4XXX_SENSOR0_IRQ_ENABLE
static volatile demo_callback_status_t  gs_irq_callback_status = DEMO_CALLBACK_STATUS_WAIT;
#endif

static volatile rm_zmod4xxx_iaq_1st_data_t      gs_iaq_1st_gen_data;
static volatile rm_zmod4xxx_iaq_2nd_data_t      gs_iaq_2nd_gen_data;
static volatile rm_zmod4xxx_odor_data_t         gs_odor_data;
static volatile rm_zmod4xxx_sulfur_odor_data_t  gs_sulfur_odor_data;

void zmod4xxx_comms_i2c_callback(rm_zmod4xxx_callback_args_t * p_args)
{
    if (RM_ZMOD4XXX_EVENT_ERROR != p_args->event)
    {
        gs_i2c_callback_status = DEMO_CALLBACK_STATUS_SUCCESS;
    }
    else
    {
        gs_i2c_callback_status = DEMO_CALLBACK_STATUS_REPEAT;
    }
}

/* TODO: Enable if you want to use a IRQ callback */
void zmod4xxx_irq_callback(rm_zmod4xxx_callback_args_t * p_args)
{
#if G_ZMOD4XXX_SENSOR0_IRQ_ENABLE
    FSP_PARAMETER_NOT_USED(p_args);

    gs_irq_callback_status = DEMO_CALLBACK_STATUS_SUCCESS;
#else
    FSP_PARAMETER_NOT_USED(p_args);
#endif
}

/* Quick setup for g_zmod4xxx_sensor0. */
void g_zmod4xxx_sensor0_quick_setup(void)
{
    fsp_err_t err;

    /* Open ZMOD4XXX sensor instance, this must be done before calling any ZMOD4XXX API */
    err = g_zmod4xxx_sensor0.p_api->open(g_zmod4xxx_sensor0.p_ctrl, g_zmod4xxx_sensor0.p_cfg);
    if (FSP_SUCCESS != err)
    {
        demo_err();
    }
}

/* Quick setup for g_comms_i2c_bus0. */
void g_comms_i2c_bus0_quick_setup(void)
{
    fsp_err_t err;
    i2c_master_instance_t * p_driver_instance = (i2c_master_instance_t *) g_comms_i2c_bus0_extended_cfg.p_driver_instance;

    /* Open I2C driver, this must be done before calling any COMMS API */
    err = p_driver_instance->p_api->open(p_driver_instance->p_ctrl, p_driver_instance->p_cfg);
    if (FSP_SUCCESS != err)
    {
        demo_err();
    }
}

/************************************************************************************
* Name:       mcu_hw_init
* Function:   initialize hardware including uart and irq
* Parameters: none
* Return:     none
************************************************************************************/
void mcu_hw_init(void)
{
    fsp_err_t   status = FSP_SUCCESS;

    /* Timer initialize */
    status = g_timer.p_api->open(g_timer.p_ctrl, g_timer.p_cfg);
    if(status)
    {
        APP_PRINT("Timer initialization failed!\r\n");
        demo_err();
    }
    else
    {
        APP_PRINT("Timer initializes successfully!\r\n");
    }
    status = g_timer.p_api->start(g_timer.p_ctrl);
    if(status)
    {
        APP_PRINT("Timer starts Error!\r\n");
        demo_err();
    }
    else
    {
        APP_PRINT("Timer starts successfully!\r\n");
    }
    /* UART for DA16200 AT command initialize */
    status = g_wifi_uart.p_api->open(g_wifi_uart.p_ctrl, g_wifi_uart.p_cfg);
    if(status)
    {
        APP_PRINT("UART initialization failed!\r\n");
        demo_err();
    }
    else
    {
        APP_PRINT("UART initializes successfully!\r\n");
    }
    /*
    do
    {
        status = g_wifi_uart.p_api->read(g_wifi_uart.p_ctrl, &data, 1);
        while((!g_wifi_rcv_flag) && (g_wifi_rcv_timeout_cnt < 1000));
        if(g_wifi_rcv_flag)
        {
            rcv_buf[cnt++] = data;
        }
        else
            break;
    }while(1);
    */
}

char str[100] = {0};

void start_demo(void)
{
    fsp_err_t               err;
    rm_zmod4xxx_raw_data_t  raw_data;
    demo_sequence_t         sequence = DEMO_SEQUENCE_1;
    rm_zmod4xxx_lib_type_t  lib_type = g_zmod4xxx_sensor0_extended_cfg.lib_type;

    /* Clear status */
    gs_i2c_callback_status = DEMO_CALLBACK_STATUS_WAIT;
#if G_ZMOD4XXX_SENSOR0_IRQ_ENABLE
    gs_irq_callback_status = DEMO_CALLBACK_STATUS_WAIT;
#endif

    /* Initialize UART for AT command transmission, Timer for 1ms counter */
    mcu_hw_init();
    /* Connect MCU with Wi-Fi module and initialize Wi-Fi module */
    err = wifi_con_init();
    if(err)
    {
        demo_err();
    }
    /* Configure Wi-Fi module to station mode and let it connect to AP */
    err = wifi_con_routine();
    if(err)
    {
        demo_err();
    }
    /* Configure Wi-Fi module to MQTT client function */
    err = mqtt_con_routine();
    if(err)
    {
        demo_err();
    }
    GREEN_LED_ON;

    /* Open the Bus */
    g_comms_i2c_bus0_quick_setup();

    /* Open ZMOD4XXX */
    g_zmod4xxx_sensor0_quick_setup();

    while(1)
    {
        switch(sequence)
        {
            case DEMO_SEQUENCE_1 :
            {
                /* Clear status */
                gs_i2c_callback_status = DEMO_CALLBACK_STATUS_WAIT;

                /* Start measurement */
                err = g_zmod4xxx_sensor0.p_api->measurementStart(g_zmod4xxx_sensor0.p_ctrl);
                if (FSP_SUCCESS == err)
                {
                    sequence = DEMO_SEQUENCE_2;
                }
                else
                {
                    demo_err();
                }
            }
            break;

            case DEMO_SEQUENCE_2 :
            {
                /* Check I2C callback status */
                switch (gs_i2c_callback_status)
                {
                    case DEMO_CALLBACK_STATUS_WAIT :
                        break;
                    case DEMO_CALLBACK_STATUS_SUCCESS :
                        sequence = DEMO_SEQUENCE_3;
                        break;
                    case DEMO_CALLBACK_STATUS_REPEAT :
                        sequence = DEMO_SEQUENCE_1;
                        break;
                    default :
                        demo_err();
                        break;
                }
            }
            break;

#if G_ZMOD4XXX_SENSOR0_IRQ_ENABLE
            case DEMO_SEQUENCE_3 :
            {
                /* Check IRQ callback status */
                switch (gs_irq_callback_status)
                {
                    case DEMO_CALLBACK_STATUS_WAIT :
                        break;
                    case DEMO_CALLBACK_STATUS_SUCCESS :
                        gs_irq_callback_status = DEMO_CALLBACK_STATUS_WAIT;
                        sequence = DEMO_SEQUENCE_5;
                        break;
                    default :
                        demo_err();
                        break;
                }
            }
            break;
#else
            case DEMO_SEQUENCE_3 :
            {
                /* Clear status */
                gs_i2c_callback_status = DEMO_CALLBACK_STATUS_WAIT;

                /* Get status */
                err = g_zmod4xxx_sensor0.p_api->statusCheck(g_zmod4xxx_sensor0.p_ctrl);
                if (FSP_SUCCESS == err)
                {
                    sequence = DEMO_SEQUENCE_4;
                }
                else
                {
                    demo_err();
                }
            }
            break;

            case DEMO_SEQUENCE_4 :
            {
                /* Check I2C callback status */
                switch (gs_i2c_callback_status)
                {
                    case DEMO_CALLBACK_STATUS_WAIT :
                        break;
                    case DEMO_CALLBACK_STATUS_SUCCESS :
                        sequence = DEMO_SEQUENCE_5;
                        break;
                    case DEMO_CALLBACK_STATUS_REPEAT :
                        sequence = DEMO_SEQUENCE_3;
                        break;
                    default :
                        demo_err();
                        break;
                }
            }
            break;
#endif
            case DEMO_SEQUENCE_5 :
            {
                /* Clear status */
                gs_i2c_callback_status = DEMO_CALLBACK_STATUS_WAIT;

                /* Read data */
                err = g_zmod4xxx_sensor0.p_api->read(g_zmod4xxx_sensor0.p_ctrl, &raw_data);
                if (FSP_SUCCESS == err)
                {
                    sequence = DEMO_SEQUENCE_6;
                }
                else if (FSP_ERR_SENSOR_MEASUREMENT_NOT_FINISHED == err)
                {
                    sequence = DEMO_SEQUENCE_3;

                    /* Delay 50ms */
                    R_BSP_SoftwareDelay(50, BSP_DELAY_UNITS_MILLISECONDS);
                }
                else
                {
                    demo_err();
                }
            }
            break;

            case DEMO_SEQUENCE_6 :
            {
                /* Check I2C callback status */
                switch (gs_i2c_callback_status)
                {
                    case DEMO_CALLBACK_STATUS_WAIT :
                        break;
                    case DEMO_CALLBACK_STATUS_SUCCESS :
                        sequence = DEMO_SEQUENCE_7;
                        break;
                    case DEMO_CALLBACK_STATUS_REPEAT :
                        sequence = DEMO_SEQUENCE_5;
                        break;
                    default :
                        demo_err();
                        break;
                }
            }
            break;

            case DEMO_SEQUENCE_7 :
            {
                /* Calculate data */
                switch (lib_type)
                {
                    case RM_ZMOD4410_LIB_TYPE_IAQ_1ST_GEN_CONTINUOUS :
                    case RM_ZMOD4410_LIB_TYPE_IAQ_1ST_GEN_LOW_POWER :
                        err = g_zmod4xxx_sensor0.p_api->iaq1stGenDataCalculate(g_zmod4xxx_sensor0.p_ctrl,
                                                                               &raw_data,
                                                                               (rm_zmod4xxx_iaq_1st_data_t*)&gs_iaq_1st_gen_data);
                        break;
                    case RM_ZMOD4410_LIB_TYPE_IAQ_2ND_GEN :
                        err = g_zmod4xxx_sensor0.p_api->iaq2ndGenDataCalculate(g_zmod4xxx_sensor0.p_ctrl,
                                                                               &raw_data,
                                                                               (rm_zmod4xxx_iaq_2nd_data_t*)&gs_iaq_2nd_gen_data);
                        break;
                    case RM_ZMOD4410_LIB_TYPE_ODOR :
                        err = g_zmod4xxx_sensor0.p_api->odorDataCalculate(g_zmod4xxx_sensor0.p_ctrl,
                                                                          &raw_data,
                                                                          (rm_zmod4xxx_odor_data_t*)&gs_odor_data);
                        break;
                    case RM_ZMOD4410_LIB_TYPE_SULFUR_ODOR :
                        err = g_zmod4xxx_sensor0.p_api->sulfurOdorDataCalculate(g_zmod4xxx_sensor0.p_ctrl,
                                                                                &raw_data,
                                                                                (rm_zmod4xxx_sulfur_odor_data_t*)&gs_sulfur_odor_data);
                        break;
                    default :
                        demo_err();
                        break;
                }

                if (FSP_SUCCESS == err)
                {
                    /* Gas data is valid. Describe the process by referring to each calculated gas data. */
                }
                else if (FSP_ERR_SENSOR_IN_STABILIZATION == err)
                {
                    /* Gas data is invalid. Sensor is in stabilization. */
                }
                else
                {
                    demo_err();
                }

                sequence = DEMO_SEQUENCE_8;
            }
            break;

            case DEMO_SEQUENCE_8 :
            {
                switch (lib_type)
                {
                    case RM_ZMOD4410_LIB_TYPE_IAQ_1ST_GEN_CONTINUOUS :
                    case RM_ZMOD4410_LIB_TYPE_ODOR :
                        sequence = DEMO_SEQUENCE_3;
                        break;
                    case RM_ZMOD4410_LIB_TYPE_IAQ_1ST_GEN_LOW_POWER :
                        /* See Table 3 in the ZMOD4410 Programming Manual. */
                        R_BSP_SoftwareDelay(5475, BSP_DELAY_UNITS_MILLISECONDS);
                        sequence = DEMO_SEQUENCE_1;
                        break;
                    case RM_ZMOD4410_LIB_TYPE_IAQ_2ND_GEN :
                    case RM_ZMOD4410_LIB_TYPE_SULFUR_ODOR :
                        /* IAQ 2nd Gen : See Table 4 in the ZMOD4410 Programming Manual. */
                        /* Sulfur Odor : See Table 6 in the ZMOD4410 Programming Manual. */
                        g_zmod_timing = 0;
                        mqtt_msg_send();
                        while(!zmod_start_flag);
                        sequence = DEMO_SEQUENCE_1;
                        zmod_start_flag = false;
                        break;
                    default :
                        demo_err();
                        break;
                }
            }
            break;

            default :
            {
                demo_err();
            }
            break;
        }
    }
}

void mqtt_msg_send(void)
{
    fsp_err_t err;

    update_flag++;
    update_flag %= 2;
    /* Clear string and set the original value */
    memset(tvoc_title, 0 , sizeof(tvoc_title));
    memcpy(tvoc_title, "TVOC:", 5);
    memset(eco2_title, 0 , sizeof(eco2_title));
    memcpy(eco2_title, "ECO2:", 5);
    memset(iaq_title, 0 , sizeof(iaq_title));
    memcpy(iaq_title, "IAQ:", 4);
    memset(air_str, 0 , sizeof(air_str));
    sprintf(tvoc_value_str, "%f", gs_iaq_2nd_gen_data.tvoc);
    strcat(tvoc_title, tvoc_value_str);
    strcat(tvoc_title, " ");
    sprintf(eco2_value_str, "%f", gs_iaq_2nd_gen_data.eco2);
    strcat(eco2_title, eco2_value_str);
    strcat(eco2_title, " ");
    sprintf(iaq_value_str, "%f", gs_iaq_2nd_gen_data.iaq);
    strcat(iaq_title, iaq_value_str);
    strcat(air_str,tvoc_title);
    strcat(air_str,eco2_title);
    strcat(air_str,iaq_title);
    g_da16200_cmd_set[DA16200_AT_CMD_INDEX_AT_NWMQMSG].p_cmd = (uint8_t *)mqtt_head;

    part_array[0] = (uint32_t)air_str;
    part_array[1] = (uint32_t)at_topic;
    part_array[2] = (uint32_t)at_cmd_end;
    At_cmd_combine(DA16200_AT_CMD_INDEX_AT_NWMQMSG, part_array, 3);
    err = AT_cmd_send_ok(DA16200_AT_CMD_INDEX_AT_NWMQMSG);
    if(err)
    {
        APP_PRINT("Communication error!\r\n");
        demo_err();
    }
    else
    {
        APP_PRINT("Air condition values are updated!\r\n");
        if(update_flag)
        {
            BLUE_LED_ON;
        }
        else
        {
            BLUE_LED_OFF;
        }
    }

}

void demo_err(void)
{
    while(1)
    {
        // nothing
    }
}
